/***************************************************************************
                          myconfig.cpp  -  description
                             -------------------
    begin                : Tue Oct 9 2001
    copyright            : (C) 2001 by Fredrik Srensson
    email                : sorn73@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "myconfig.h"

myconfig::myconfig( QWidget* parent,const char* name,bool modal,WFlags fl) :
		ldapctrldlg(parent,name,modal,fl)
{

	// This line seems to make the list unsorted, just as we want it. I give no guarantee though.	
	this->schemeView->setSorting(-1, true);
	entryUpdate=true;
	this->defaultScheme->setChecked(true);

	workConf = new KSimpleConfig("myldapklientrc",false);
	if (!workConf->hasGroup("Global")) {
		workConf->setGroup("Global");
		workConf->writeEntry("lastsearch","");
		return;
	};

	initServers();
};


//---------------------------------------------------------------------
// Initialize serverlist. Helper to the constructor.

void myconfig::initServers() {
	QString server;

// Let's make an educated guess about which server to select in the listbox

	workConf->setGroup("Global");
	if (workConf->hasKey("lastserver") ) {
		server=workConf->readEntry("lastserver");
	} else {
		workConf->setGroup("Servers");
		if (workConf->hasKey("server0") ) {
			server=workConf->readEntry("server0");
		} else {
			server ="";
		}
	}

  buildServList(server);
	loadServer(server);
}

//---------------------------------------------------------------------
// Build the ServList entries from the [Servers] config group. Select the current server if we have one

void myconfig::buildServList(QString & inServer) {
	QString key,thisServer;
	int i;

	this->ldapConfServSel->clear();

	if (!workConf->hasGroup("Servers")) {
		return;
	};
	workConf->setGroup("Servers");
	for (i=0; i<LAMEMAX; i++) { 			// Lame. Should terminate when exhausted, and not max out at LAMEMAX.
		key="";
		QTextOStream(&key) << "server" << i;
		if (workConf->hasKey(key)) {
			thisServer=workConf->readEntry(key);
			this->ldapConfServSel->insertItem( thisServer , -1);
			if (thisServer == inServer) {this->ldapConfServSel->setCurrentItem(i); };
		} else {
			break;
		}
	}
	lastserver=inServer;
}


//---------------------------------------------------------------------
// Load a new server and update the GUI with all the values.

void myconfig::loadServer(QString & inServer) {

	QString group,key,head,kabtag,ldapattr;
	QListViewItem *dLvi;
	QListViewItem *lastItem;
	int i;

	this->schemeView->clear();	

	if (inServer == NULL) {return;};

	group ="";
	lastItem=NULL;
	QTextOStream(&group) << "Server" << inServer;

	workConf->setGroup(group);

	this->ldapServer->setText(workConf->readEntry("ldaphost"));
	this->ldapBase->setText(workConf->readEntry("ldapbase"));
	this->ldapDN->setText(workConf->readEntry("dn"));
	this->ldapPassword->setText(workConf->readEntry("password"));
	this->ldapFilter->setText(workConf->readEntry("ldapfilter"));

  if (workConf->readEntry("ldapversion") == "3" ) {
    	this->ldapVersion->setCurrentItem(1);
  } else {
    	this->ldapVersion->setCurrentItem(0);
  }

	for (i=0; i<LAMEMAX; i++) {					// Still this darned lame loop.
		key="";
		QTextOStream(&key)<<"head"<<i;
		if (workConf->hasKey(key)) {
			head=workConf->readEntry(key);
		} else {
			break;
		}
		key="";
		QTextOStream(&key)<<"ldapattr"<<i;
		if (workConf->hasKey(key)) {
			ldapattr=workConf->readEntry(key);
		} else {
			break;
		}
		key="";
		QTextOStream(&key)<<"kabtag"<<i;
		if (workConf->hasKey(key)) {
			kabtag=workConf->readEntry(key);
		} else {
			break;
		}

		key="";
		QTextOStream(&key) << i;

		dLvi = new QListViewItem (this->schemeView,lastItem,ldapattr,kabtag,head);
		lastItem=dLvi;

		clearEntry();
    slot_selSchemeEntry();
	}
}



//---------------------------------------------------------------------
// Save the "last" server, so that when you change something in a scheme, and then change server
// and then change back to the original server, changes won't get lost.
// However, if you delete an entry, the lastServer is still poting to the now deleted server. That's why
// some members clear lastServer. Maybe they should NULL it. Anyway, we check for both.

void myconfig::saveServer(QString & inServer) {
	
	QString group, key;
	int i;
	QListViewItemIterator iter(this->schemeView);

	if (inServer == NULL) { return; }		// Note to self: Learn c++ syntax to see how to collapse these lines
	if (inServer == "") { return; }     // to one line
	
	group = "";
	QTextOStream(&group) << "Server" << inServer;
	
	workConf->deleteGroup(group);
	workConf->setGroup (group);
	workConf->writeEntry("ldaphost",this->ldapServer->text() );
	workConf->writeEntry("ldapbase",this->ldapBase->text() );
	workConf->writeEntry("ldapfilter",this->ldapFilter->text() );
	workConf->writeEntry("dn",this->ldapDN->text() );
	workConf->writeEntry("password",this->ldapPassword->text() );

  if (this->ldapVersion->currentItem() == 0 ) {
    workConf->writeEntry("ldapversion","2" );
  } else {
    workConf->writeEntry("ldapversion","3" );
  }

	for (i=0; i<LAMEMAX; i++) {					// Still this darned lame loop.

		if (iter.current() == NULL ) {
			break;
		} else {
			key="";
			QTextOStream(&key)<<"head"<<i;
			workConf->writeEntry(key,iter.current()->text(2) );

			key="";
			QTextOStream(&key)<<"ldapattr"<<i;
			workConf->writeEntry(key,iter.current()->text(0) );

			key="";
			QTextOStream(&key)<<"kabtag"<<i;
			workConf->writeEntry(key,iter.current()->text(1) );
			iter++;
		}
	}
}

//---------------------------------------------------------------------
// User selected a new server, update view.

void myconfig::slot_newServer() {
	QString server;
	
	saveServer(lastserver);
	server=this->ldapConfServSel->currentText();
	lastserver=server;
	loadServer(server);

}

//---------------------------------------------------------------------
// User deletes a server, and we are only so happy to oblige
void myconfig::slot_deleteServer() {
	QString delServ,anServ;
  int currI;

	currI = this->ldapConfServSel->currentItem();
  delServ = this->ldapConfServSel->currentText();

	this->ldapConfServSel->removeItem(currI);

	if ( delServ == lastserver) {
		this->schemeView->clear();					
    clearEntry();
		this->ldapServer->clear();
		this->ldapBase->clear();
	}

	workConf->deleteGroup("Server"+delServ);
	lastserver="";      // so we don't save a deleted server.

	// If a server was selected in the main dialog AND we deleted it in the
	// conf dialog AND returned back to the main dialog, you got an error
	// message saying that that server didn't exist. Which was true, but
	// annoying. The snippet below fixes that.

	workConf->setGroup("Global");
	if ( (workConf->hasKey("lastserver")) && (workConf->readEntry("lastserver") == delServ) ) {
		workConf->deleteEntry("lastserver");
	}

}

//---------------------------------------------------------------------
// User wan'ts to add a server, and so we will.

void myconfig::slot_addServer() {
	QString message,newServer;

	newServer=this->ldapServName->text();

	if ( newServer == "" ) { return; }

	if ( this->ldapConfServSel->findItem(newServer) != NULL) {
		message="";
		QTextOStream(&message)<<"Server "<<newServer<<" already exists.";
		KMessageBox::error(this,message);
		return;
	}

	workConf->setGroup("Server"+newServer);
	workConf->writeEntry("ldapbase","o=");

	this->ldapServName->setText("");

	this->ldapConfServSel->insertItem(newServer);			// Aagh! Why doesn't insertItem return where it inserted it?
	this->ldapConfServSel->setCurrentItem(this->ldapConfServSel->findItem(newServer)); 	// Eww!
	slot_newServer();	
	if (this->defaultScheme->isChecked() ) { setDefaultScheme(); };
}


//---------------------------------------------------------------------
// Save the list of servers and return the number of servers in the list.

int myconfig::saveServerList() {
	QString key;
	unsigned int i;
	i=0;
	workConf->deleteGroup("Servers");
	workConf->setGroup("Servers");

	for (i=0;i < this->ldapConfServSel->count();i++) {
		key="";
		QTextOStream(&key)<<"server"<<i;
		workConf->writeEntry(key,this->ldapConfServSel->text(i));
	}					
	lastserver="";
	return (this->ldapConfServSel->count() -1);
}

//---------------------------------------------------------------------
// User clicked OK, we'll save and we're outta here!
void myconfig::accept() {
	saveServer(lastserver);
	saveServerList();
	workConf->sync();
	this->close(true);
}

//
// Here is a line between members for the "upper" and "lower" part of the GUI
//

//---------------------------------------------------------------------
// Apply a default X500/rfc 2256 scheme

void myconfig::setDefaultScheme() {
	QListViewItem * lastItem;

	this->schemeView->clear();
	
	lastItem = new QListViewItem (this->schemeView,"mail","emails","Email");	
	lastItem = new QListViewItem (this->schemeView,lastItem,"cn","fn","Full Name");
	lastItem = new QListViewItem (this->schemeView,lastItem,"sn","lastname","Last Name");
	lastItem = new QListViewItem (this->schemeView,lastItem,"givenName","firstname","First Name");
	lastItem = new QListViewItem (this->schemeView,lastItem,"title","title","Title");

	lastItem = new QListViewItem (this->schemeView,lastItem,"l","Work/town","Town");
	lastItem = new QListViewItem (this->schemeView,lastItem,"st","Work/state","State");
	lastItem = new QListViewItem (this->schemeView,lastItem,"street","Work/address","Address");

	lastItem = new QListViewItem (this->schemeView,lastItem,"telephoneNumber","phone/Fixed","Telephone");
	lastItem = new QListViewItem (this->schemeView,lastItem,"facsimileTelephoneNumber","phone/Fax","Fax");

}


//---------------------------------------------------------------------
// In a "safe" fashion, clear the ldapattr, kabtag and heading fields

void myconfig::clearEntry() {
	entryUpdate=false;
	this->ldapattr->clear();
	this->kabtag->setCurrentItem(0);
	this->heading->clear();
	entryUpdate=true;
}

//---------------------------------------------------------------------
// Just a little helper member, nothing to see here, move along

int myconfig::kabtagNum(QString instr) {
	int comboCount = (this->kabtag->count()) +1;
	int i;

	for (i=0; i<comboCount; i++) {
		if (instr == this->kabtag->text(i) ) { return i;}
	}
	return 0;
}

//---------------------------------------------------------------------
// User selected an entry in the scheme

void myconfig::slot_selSchemeEntry() {

	if (this->schemeView->currentItem() == NULL) { return; }

	entryUpdate=false;
	this->ldapattr->setText(this->schemeView->currentItem()->text(0));
	this->kabtag->setCurrentItem(kabtagNum(this->schemeView->currentItem()->text(1)));
	this->heading->setText(this->schemeView->currentItem()->text(2));
	entryUpdate=true;
}

//---------------------------------------------------------------------
// User just updated something, get it into the list
// However, not just user action triggers these slots. When I set, or clear, the textfields
// elsewhere, these slots get signals. That is a problem. However, I think I have worked around it.

void myconfig::slot_updateLdapattr() {
	if (entryUpdate == false) {return;}
	if (this->schemeView->currentItem() == NULL) { return; }
	this->schemeView->currentItem()->setText(0, this->ldapattr->text() );
}

void myconfig::slot_updateKabtag() {
	if (entryUpdate == false) {return;}
	if (this->schemeView->currentItem() == NULL) { return; }
	this->schemeView->currentItem()->setText(1, this->kabtag->currentText() );	
}

void myconfig::slot_updateHeading() {
	if (entryUpdate == false) {return;}
	if (this->schemeView->currentItem() == NULL) { return; }
	this->schemeView->currentItem()->setText(2, this->heading->text() );
}

//---------------------------------------------------------------------
// Delete an entry. Again, the textchanged signals caused some problems.
// Maybe I should implement a flag saying that even though the text changed, don't update the list.
// I did.

void myconfig::slot_deleteEntry() {
	if (this->schemeView->currentItem() == NULL) { return; }
	clearEntry();
	this->schemeView->takeItem(this->schemeView->currentItem());
}

//---------------------------------------------------------------------
// Add a new entry. This one also caused some problems as the list suddenly would sort
// which was not diserable. I think I got it fixed.

void myconfig::slot_addEntry() {
	QListViewItemIterator iter(this->schemeView);
	QListViewItem * lastItem;
	QListViewItem * dLvi;

	for ( ; iter.current(); ++iter ) {		// If there is a better way of adding a listview item last in a list,
		lastItem = iter.current();          // please tell me.
	}
		
	dLvi = new QListViewItem (this->schemeView,lastItem,"new entry","emails","New Entry");
	this->schemeView->setSelected(dLvi, true);

}

void myconfig::slot_down() {
	QListViewItem * lastItem;

	if (this->schemeView->currentItem() == NULL) { return; }

	lastItem= this->schemeView->currentItem();
	if (lastItem->itemBelow() != NULL) {
		lastItem->moveItem(lastItem->itemBelow() );
	}
}

void myconfig::slot_up() {
	QListViewItem * lastItem;

	if (this->schemeView->currentItem() == this->schemeView->firstChild() ) { return; }
	lastItem= this->schemeView->currentItem();
	if (lastItem->itemAbove() == this->schemeView->firstChild() ) {
		this->schemeView->takeItem(lastItem);
		this->schemeView->insertItem(lastItem);
		this->schemeView->setSelected(lastItem,true);
		return;
	}
	if (lastItem->itemAbove()->itemAbove() != NULL) {
		lastItem->moveItem(lastItem->itemAbove()->itemAbove() );
	}

}	

//---------------------------------------------------------------------
// Simple destructor.

myconfig::~myconfig() {
	delete workConf;
};

